//+------------------------------------------------------------------+
//|                                                          RVI.mq4 |
//|                                                                  |
//+------------------------------------------------------------------+
#property  copyright ""
#property  link      ""

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  clrOrange
#property indicator_color2  clrLimeGreen
#property indicator_color3  clrLimeGreen
#property indicator_color4  clrRed
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2

extern ENUM_TIMEFRAMES TimeFrame   = PERIOD_CURRENT;
extern int             RVIPeriod   = 10;
extern bool            Interpolate = true;

//
//
//
//
//

double rvi[];
double rviua[];
double rviub[];
double sig[];
double buffer1[];
double buffer2[];
double slope[];
string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(7);
   SetIndexBuffer(0,rvi);
   SetIndexBuffer(1,rviua);
   SetIndexBuffer(2,rviub);
   SetIndexBuffer(3,sig);
   SetIndexBuffer(4,buffer1);
   SetIndexBuffer(5,buffer2);
   SetIndexBuffer(6,slope);
   SetIndexDrawBegin(0,RVIPeriod+3);   
   SetIndexDrawBegin(1,RVIPeriod+3);   
   SetIndexDrawBegin(2,RVIPeriod+3);   
   SetIndexDrawBegin(3,RVIPeriod+7); 
   
   indicatorFileName = WindowExpertName();
   returnBars        = TimeFrame==-99;
   TimeFrame         = MathMax(TimeFrame,_Period);  
   
   IndicatorShortName(timeFrameToString(TimeFrame)+" RVI ("+RVIPeriod+")");
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { rvi[0] = limit+1; return(0); }
   
   //
   //
   //
   //
   //
   
   if (TimeFrame == Period())
   {
     if (slope[limit]==1) CleanPoint(limit,rviua,rviub);
     for(int i=limit; i>=0; i--)
     {
        buffer1[i] = ((Close[i]-Open[i])+2*(Close[i+1]-Open[i+1])+2*(Close[i+2]-Open[i+2])+(Close[i+3]-Open[i+3]))/6;
        buffer2[i] = ((High[i]-Low[i])  +2*(High[i+1]-Low[i+1])  +2*(High[i+2]-Low[i+2])  +(High[i+3]-Low[i+3])  )/6;
      
        //
        //
        //
        //
        //
      
        double num   = 0.0; 
        double denom = 0.0;
           for(int k=0; k<RVIPeriod; k++)
           {
              num   += buffer1[i+k];
              denom += buffer2[i+k];
           }
           if(denom!=0.0)
                 rvi[i] = num/denom;
           else  rvi[i] = 0.0;
           sig[i] = (rvi[i]+2*rvi[i+1]+2*rvi[i+2]+rvi[i+3])/6;
         
           //
           //
           //
           //
           //
         
           slope[i] = slope[i+1];
           rviua[i] = EMPTY_VALUE;
           rviub[i] = EMPTY_VALUE;
           if (rvi[i]>rvi[i+1]) slope[i] = 1;
           if (rvi[i]<rvi[i+1]) slope[i] =-1;
           if (slope[i] == 1) PlotPoint(i,rviua,rviub,rvi);
     }   
  return(0);
  }
  
  //
  //
  //
  //
  //
  
  limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
  if (slope[limit]==-1) CleanPoint(limit,rviua,rviub);
  for (i=limit;i>=0; i--)
  {
     int y = iBarShift(NULL,TimeFrame,Time[i]);
        rvi[i]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,RVIPeriod,0,y);
        rviua[i] = EMPTY_VALUE;
        rviub[i] = EMPTY_VALUE;
        sig[i]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,RVIPeriod,3,y);
        slope[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,RVIPeriod,6,y);
          
          //
          //
          //
          //
          //
      
          if (!Interpolate || y==iBarShift(NULL,TimeFrame,Time[i-1])) continue;

          //
          //
          //
          //
          //

          datetime time = iTime(NULL,TimeFrame,y);
             for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
             for(int s = 1; s < n; s++) 
             {
                rvi[i+s] = rvi[i] + (rvi[i+n] - rvi[i]) * s/n;
                sig[i+s] = sig[i] + (sig[i+n] - sig[i]) * s/n; 
             }                        
      }
      for (i=limit;i>=0;i--) if (slope[i] == 1) PlotPoint(i,rviua,rviub,rvi);
return(0);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i];  first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] =  from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                           second[i] = EMPTY_VALUE; }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

